/*****************************************************************************
 * commands.c: lirc commands handling
 *****************************************************************************
 * $Id: commands.c,v 1.6 2003/05/02 11:59:51 keuleu Exp $
 *****************************************************************************
 * Copyright (C) 2001 Keuleu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************
 *
 * This software was based on xawtv. Those portions are
 * Copyright (C) 2000 Gerd Knorr <kraxel@cs.tu-berlin.de>
 *
 *****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <math.h>
#include <stdarg.h>
#include <time.h>

#include <X11/Intrinsic.h>

#include "config.h"
#include "commands.h"
#include "channel.h"
#include "plugin.h"

/* ----------------------------------------------------------------------- */
extern int debug;
extern Widget app_shell;
extern PLUGIN *plugin;

void SetChannelAction (Widget, XEvent *, String *, Cardinal *);
void VolumeAction (Widget, XEvent *, String *, Cardinal *);
void FullScreenAction (Widget, XEvent *, String *, Cardinal *);
void SetResAction (Widget, XEvent *, String *, Cardinal *);
#if defined(HAVE_DIVX4LINUX) || defined(HAVE_FFMPEG) || defined(HAVE_XVID)
  void RecordMovieAction (Widget, XEvent *, String *, Cardinal *);
#endif
void ShowtimeAction (Widget, XEvent *, String *, Cardinal *);
void AleVTAction (Widget, XEvent *, String *, Cardinal *);
void ExitCB (Widget widget, XtPointer client_data, XtPointer calldata);


/* ----------------------------------------------------------------------- */

int get_arg(char **arg)
{
  int n;
  char *tok;

  if ((tok=strtok(NULL, " ")) != NULL)
    n = 1;
  else
    n = 0;

  *arg = tok;
  
  return n;
}

int
do_command(char *lirc_cmd)
{
    char *tok;
    int   argc;
    char *cmd;

    cmd = malloc(strlen(lirc_cmd)+1);
    strcpy(cmd, lirc_cmd);

    for (tok = strtok(cmd, " "); tok != (char *)NULL; tok=strtok(NULL, " "))
      {
        if (strcmp(tok, "setstation") == 0)
          {
            argc = get_arg(&tok); // next, prev
            SetChannelAction(NULL, NULL, &tok, &argc);
          }
        else if  (strcmp(tok, "volume") == 0)
          {
            argc = get_arg(&tok); // inc, dec, mute or numeric value
            VolumeAction(NULL, NULL, &tok, &argc);
          }
        else if  (strcmp(tok, "fullscreen") == 0)
          {
            argc = get_arg(&tok); // toggle
            FullScreenAction(NULL, NULL, &tok, &argc);
          }
        else if  (strcmp(tok, "setres") == 0)
          {
            char *argv[2];
            argc = get_arg(&tok); // capture
            argv[0] = tok;
            argc += get_arg(&tok); // toggle
            argv[1] = tok;
            SetResAction (NULL, NULL, argv, &argc);
          }
        else if  (strcmp(tok, "showtime") == 0)
          {
            // no args
            argc = 0;
            ShowtimeAction (NULL, NULL, NULL, NULL);
          }
        else if  (strcmp(tok, "quit") == 0)
          {
            // no args
            argc = 0;
            ExitCB (app_shell, NULL, NULL);
          }
        else if  (strcmp(tok, "keypad") == 0)
          {
            argc = get_arg(&tok); // [0..9]
            SetChannelAction(NULL, NULL, &tok, &argc);
          }
#if defined(HAVE_DIVX4LINUX) || defined(HAVE_FFMPEG) || defined(HAVE_XVID)
        else if  (strcmp(tok, "record") == 0)
          {
            // no args
            argc = 0;
            RecordMovieAction(NULL, NULL, NULL, &argc);
          }
#endif
        else if  (strcmp(tok, "teletext") == 0)
          {
            // no args
            argc = 0;
            AleVTAction(NULL, NULL, NULL, &argc);
          }
        else
          {
            int ret = -1;
            if (plugin)
              ret = plugin->do_command(&tok);
            if (ret == -1)
              fprintf (stderr, "lirc: unknown command: %s\n", tok);
          }
      }

    free(cmd);

    return 0;
}

