/*****************************************************************************
 * divx_ui.c: DivX Parameters editor
 *****************************************************************************
 * $Id: divx_ui.c,v 1.5 2003/02/06 07:58:34 keuleu Exp $
 *****************************************************************************
 * Copyright (C) 2001 Keuleu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <sys/time.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/XawInit.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Scrollbar.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>

#include "config.h"

#if defined(HAVE_DIVX4LINUX) || defined(HAVE_FFMPEG) || defined (HAVE_XVID)
#include "divx.h"
#include "strtab.h"
#include "toolbox.h"
#include "xawdecode.h"

/*-------------------------------------------------------------------------*/

extern Widget app_shell, divx_shell;
extern Display *dpy;
extern Atom wm_protocols[2];
extern int debug;

extern divx_t divx;

static int w_modul = 32;
static int h_modul = 16;
static Widget divx_shell;
static Widget divx_paned;
static Widget c_divxcodec, c_wmodul, c_hmodul;
static Widget l_divxwidth,        s_divxwidth;
static Widget l_divxheight,       s_divxheight;
static Widget l_divxbitrate,      s_divxbitrate;
static Widget c_divxquality;
static Widget l_divxminquantizer, s_divxminquantizer;
static Widget l_divxmaxquantizer, s_divxmaxquantizer;
static Widget l_divxaudiobufsize, s_divxaudiobufsize;
static Widget t_divxfilepath;
#ifdef HAVE_LAME
static Widget c_compressaudio;
static Widget p_divxmp3bitrate,   l_divxmp3bitrate,   s_divxmp3bitrate;
static Widget p_divxmp3quality,   l_divxmp3quality,   s_divxmp3quality;
#endif // HAVE_LAME
static Widget c_displayframes;

enum {
  DIVX_CODEC = 10,
  DIVX_HMODUL,
  DIVX_WMODUL,
  DIVX_QUALITY,
  DIVX_COMPRESS_AUDIO,
  DIVX_DISPLAY_FRAMES
};

static struct STRTAB on_off_list[] = {
  {1, "On"},
  {0, "Off"},
  {-1, NULL}
};

static struct STRTAB codec_list[] = {
#ifdef HAVE_DIVX4LINUX
  {DIVX4LINUX, "DivX"},
#endif
#ifdef HAVE_FFMPEG
  {FFMPEG_MPEG4, "FFMPEG - Mpeg4"},
  {FFMPEG_MPEG1, "FFMPEG - Mpeg1Video"},
#endif
#ifdef HAVE_XVID
  {XVID, "XviD"},
#endif
  {-1, NULL}
};

static struct STRTAB modulo_list[] = {
  {1,  " 1"},
  {2,  " 2"},
  {4,  " 4"},
  {8,  " 8"},
  {16, "16"},
  {32, "32"},
  {-1, NULL}
};

#ifdef HAVE_DIVX4LINUX
static struct STRTAB divx4linux_quality_list[] = {
  {1,  "Fastest"},
  {2,  "Fast"},
  {3,  "Medium"},
  {4,  "Slow"},
  {5,  "Slowest"},
  {-1, NULL}
};
#endif

#ifdef HAVE_FFMPEG
static struct STRTAB ffmpeg_quality_list[] = {
  {0,  "No ME"},
  {1,  "Very Low"},
  {2,  "Low"},
  {3,  "Medium"},
  {4,  "High"},
  {5,  "Very High"},
  {-1, NULL}
};
#endif

#ifdef HAVE_XVID
static struct STRTAB xvid_quality_list[] = {
  {0,  "H623, No ME"},
  {1,  "MPEG, Low"},
  {2,  "H623, Low"},
  {3,  "H623, Medium"},
  {4,  "H623, High"},
  {5,  "H623, Very High"},
  {6,  "H623, Ultra High"},
  {-1, NULL}
};
#endif

static struct STRTAB *quality_list = NULL;

/*-------------------------------------------------------------------------*/
#define FIX_RIGHT_TOP       \
    XtNleft,XawChainRight,  \
    XtNright,XawChainRight, \
    XtNtop,XawChainTop,     \
    XtNbottom,XawChainTop

#define DIVX_WIDTH_RANGE         768.0
#define DIVX_HEIGHT_RANGE        576.0
#define DIVX_BITRATE_RANGE     10000.0
#define DIVX_QUALITY_RANGE         5.0
#define DIVX_MINQUANTIZER_RANGE   64.0
#define DIVX_MAXQUANTIZER_RANGE   64.0
#define AUDIO_BUFFERSIZE_RANGE 65535.0
#ifdef HAVE_LAME
#  define MP3_BITRATE_RANGE        512.0
#  define MP3_QUALITY_RANGE          9.0
#endif // HAVE_LAME

#define LABEL_WIDTH2 "18"
/*-------------------------------------------------------------------------*/

struct STRTAB *
set_quality_list(int codec)
{
  switch (codec)
    {
#ifdef HAVE_DIVX4LINUX
    case DIVX4LINUX:
      return divx4linux_quality_list;
      break;
#endif
#ifdef HAVE_FFMPEG
    case FFMPEG_MPEG4:
    case FFMPEG_MPEG1:
      return ffmpeg_quality_list;
      break;
#endif
#ifdef HAVE_XVID
    case XVID:
      return xvid_quality_list;
      break;
#endif
    default:
      return NULL;
      break;
    }
}

void
DivXAction (Widget widget, XEvent * event,
              String * params, Cardinal * num_params)
{
  static int mapped = 0, first = 1;

  if (event && event->type == ClientMessage)
    {
      if (event->xclient.data.l[0] == wm_protocols[1])
        {
          if (debug)
            fprintf (stderr, "Config: wm_save_yourself\n");
          XSetCommand (XtDisplay (divx_shell), XtWindow (divx_shell), NULL,
                       0);
          return;
        }
    }

  if (divx_shell == NULL)
    {
      fprintf(stderr, "DivXAction: ooops: divx_shell is NULL\n");
      return;
    }

  if (mapped)
    {
      XtPopdown (divx_shell);
      mapped = 0;
    }
  else
    {
      XtPopup (divx_shell, XtGrabNone);
      mapped = 1;
      if (first)
        {
          XSetWMProtocols (XtDisplay (divx_shell), XtWindow (divx_shell),
                           wm_protocols, 2);
          first = 0;
        }
    }
}

static void
divx_menu_cb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  int cd = (int) clientdata;
  int j;
  char label[64];
  int value;

  switch (cd)
    {
      /* DivX Parameters window entries */
    case DIVX_CODEC:
      if (-1 != (j = popup_menu (widget, "Codec", codec_list)))
        {
          divx.codec = j;
          sprintf (label, "%-" LABEL_WIDTH2 "s:       %s", "Codec",
                   int_to_str (divx.codec, codec_list));
          XtVaSetValues (c_divxcodec, XtNlabel, label, NULL);
          quality_list = set_quality_list(divx.codec);
        }
      break;

    case DIVX_WMODUL:
      if (-1 != (j = popup_menu (widget, "W-Modul", modulo_list)))
        {
          w_modul = j;
          sprintf (label, "%-" LABEL_WIDTH2 "s: %5d", "W-Modul", w_modul);
          XtVaSetValues (c_wmodul, XtNlabel, label, NULL);

          // Recompute new width value
          value = divx.width / w_modul;
          value = value * w_modul;
          divx.width = value;
          if (s_divxwidth)
            set_float (s_divxwidth, XtNtopOfThumb, (float)value/DIVX_WIDTH_RANGE);

          sprintf(label, "%-" LABEL_WIDTH2 "s: %5d", "Width", divx.width);
          if (l_divxwidth)
            XtVaSetValues(l_divxwidth, XtNlabel, label, NULL);
        }
      break;

    case DIVX_HMODUL:
      if (-1 != (j = popup_menu (widget, "H-Modul", modulo_list)))
        {
          h_modul = j;
          sprintf (label, "%-" LABEL_WIDTH2 "s: %5d", "H-Modul", h_modul);
          XtVaSetValues (c_hmodul, XtNlabel, label, NULL);

          // Recompute new height value
          value = divx.height / h_modul;
          value = divx.height * h_modul;
          divx.height = value;
          if (s_divxheight)
            set_float (s_divxheight, XtNtopOfThumb, (float)value/DIVX_HEIGHT_RANGE);

          sprintf(label, "%-" LABEL_WIDTH2 "s: %5d", "Height", divx.height);
          if (l_divxheight)
            XtVaSetValues(l_divxheight, XtNlabel, label, NULL);
        }
      break;

    case DIVX_QUALITY:
      if (-1 != (j = popup_menu (widget, "Quality", quality_list)))
        {
          divx.quality = j;
          sprintf (label, "%-" LABEL_WIDTH2 "s:       %s", "Quality",
                   int_to_str (divx.quality, quality_list));
          XtVaSetValues (c_divxquality, XtNlabel, label, NULL);
        }
      break;

#ifdef HAVE_LAME
    case DIVX_COMPRESS_AUDIO:
      if (-1 != (j = popup_menu (widget, "Compress Audio", on_off_list)))
        {
          divx.compress_audio = j;
          sprintf (label, "%-" LABEL_WIDTH2 "s:       %s", "Compress Audio",
                   int_to_str (divx.compress_audio, on_off_list));
          XtVaSetValues (c_compressaudio, XtNlabel, label, NULL);
          if (j == 0)
            {
              //VaSetValues(p_divxmp3bitrate, XtNsensitive, False, NULL);
              XtVaSetValues(l_divxmp3bitrate, XtNsensitive, False, NULL);
              XtVaSetValues(s_divxmp3bitrate, XtNsensitive, False, NULL);
              //VaSetValues(p_divxmp3quality, XtNsensitive, False, NULL);
              XtVaSetValues(l_divxmp3quality, XtNsensitive, False, NULL);
              XtVaSetValues(s_divxmp3quality, XtNsensitive, False, NULL);
            }
          else
            {
              //VaSetValues(p_divxmp3bitrate, XtNsensitive, True, NULL);
              XtVaSetValues(l_divxmp3bitrate, XtNsensitive, True, NULL);
              XtVaSetValues(s_divxmp3bitrate, XtNsensitive, True, NULL);
              //VaSetValues(p_divxmp3quality, XtNsensitive, True, NULL);
              XtVaSetValues(l_divxmp3quality, XtNsensitive, True, NULL);
              XtVaSetValues(s_divxmp3quality, XtNsensitive, True, NULL);
            }
        }
      break;
#endif

    case DIVX_DISPLAY_FRAMES:
      if (-1 != (j = popup_menu (widget, "Display Frames", on_off_list)))
        {
          divx.display_frames = j;
          sprintf (label, "%-" LABEL_WIDTH2 "s:       %s", "Display Frames",
                   int_to_str (divx.display_frames, on_off_list));
          XtVaSetValues (c_displayframes, XtNlabel, label, NULL);
        }
      break;

    default:
      break;
    }
}

static void
divx_jump_scb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  char *name;
  int value = 0;
  float data;
  char label[64];

  data = *(float *) call_data;
  name = XtName (XtParent (widget));
#if 0
  fprintf (stderr, "jump to %f (%s/%d)\n", *(float *) call_data, name, value);
#endif

  if (0 == strcmp(name, "divxwidth"))
    {
      value = (int) rint(data * DIVX_WIDTH_RANGE);
      if (value == 0)
        value = w_modul; // minimun acceptable value

      // make sure width is divisible by w_modul
      value /= w_modul;
      value *= w_modul;
      divx.width = value;

      if (s_divxwidth)
        set_float (s_divxwidth, XtNtopOfThumb, (float)value/DIVX_WIDTH_RANGE);

      sprintf (label, "%-" LABEL_WIDTH2 "s: %5d", "Width", divx.width);
      //sprintf(label, "Width: %3d", divx.width);
      if (l_divxwidth)
        XtVaSetValues(l_divxwidth, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "divxheight"))
    {
      value = (int) rint(data * DIVX_HEIGHT_RANGE);
      if (value == 0)
        value = h_modul; // minimun acceptable value

      // make sure height is divisible by h_modul
      value /= h_modul;
      value *= h_modul;
      divx.height = value;

      if (s_divxheight)
        set_float (s_divxheight, XtNtopOfThumb, (float)value/DIVX_HEIGHT_RANGE);

      sprintf(label, "%-" LABEL_WIDTH2 "s: %5d", "Height", divx.height);
      if (l_divxheight)
        XtVaSetValues(l_divxheight, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "divxbitrate"))
    {
      value = (int) rint(data * DIVX_BITRATE_RANGE);
      if (value == 0)
        value = 1;
      divx.bitrate = value*1000;

      if (s_divxbitrate)
        set_float (s_divxbitrate, XtNtopOfThumb, (float)value/DIVX_BITRATE_RANGE);

      sprintf(label, "%-" LABEL_WIDTH2 "s: %5d", "Bitrate (kbit/s)", value);
      if (l_divxbitrate)
        XtVaSetValues(l_divxbitrate, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "divxminquantizer"))
    {
      value = (int) (data * DIVX_MINQUANTIZER_RANGE);
      if (value == 0)
        value = 1;
      divx.min_quantizer = value;

      if (s_divxminquantizer)
        set_float (s_divxminquantizer, XtNtopOfThumb, (float)value/DIVX_MINQUANTIZER_RANGE);

      sprintf(label, "%-" LABEL_WIDTH2 "s: %5d", "Min Quantizer", divx.min_quantizer);
      if (l_divxminquantizer)
        XtVaSetValues(l_divxminquantizer, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "divxmaxquantizer"))
    {
      value = (int) (data * DIVX_MAXQUANTIZER_RANGE);
      if (value == 0)
        value = 1;
      divx.max_quantizer = value;

      if (s_divxmaxquantizer)
        set_float (s_divxmaxquantizer, XtNtopOfThumb, (float)value/DIVX_MINQUANTIZER_RANGE);

      sprintf(label, "%-" LABEL_WIDTH2 "s: %5d", "Max Quantizer", divx.max_quantizer);
      if (l_divxmaxquantizer)
        XtVaSetValues(l_divxmaxquantizer, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "divxaudiobufsize"))
    {
      value = (int) (data * AUDIO_BUFFERSIZE_RANGE);
      if (value == 0)
        value = 441;
      divx.audio_buffer_size = value;

      if (s_divxaudiobufsize)
        set_float (s_divxaudiobufsize, XtNtopOfThumb, (float)value/AUDIO_BUFFERSIZE_RANGE);

      sprintf(label, "%-" LABEL_WIDTH2 "s: %5d", "Audio Buffer Size", divx.audio_buffer_size);
      if (l_divxaudiobufsize)
        XtVaSetValues(l_divxaudiobufsize, XtNlabel, label, NULL);
    }
#ifdef HAVE_LAME
  else if (0 == strcmp(name, "divxmp3bitrate"))
    {
      value = (int) (data * MP3_BITRATE_RANGE);
      if (value == 0)
        value = 16;
      divx.mp3_bitrate = value;

      if (s_divxmp3bitrate)
        set_float (s_divxmp3bitrate, XtNtopOfThumb, (float)value/MP3_BITRATE_RANGE);

      sprintf(label, "%-" LABEL_WIDTH2 "s: %5d", "MP3 Bitrate", divx.mp3_bitrate);
      if (l_divxmp3bitrate)
        XtVaSetValues(l_divxmp3bitrate, XtNlabel, label, NULL);
    }
  else if (0 == strcmp(name, "divxmp3quality"))
    {
      value = (int) (data * MP3_QUALITY_RANGE);
      if (value == 0)
        value = 1 ;
      divx.mp3_quality = value;

      if (s_divxmp3quality)
        set_float (s_divxmp3quality, XtNtopOfThumb, (float)value/MP3_QUALITY_RANGE);

      sprintf(label, "%-" LABEL_WIDTH2 "s: %5d", "MP3 Quality", divx.mp3_quality);
      if (l_divxmp3quality)
        XtVaSetValues(l_divxmp3quality, XtNlabel, label, NULL);
    }
#endif // HAVE_LAME

#if 0
  printf("%s: value = %d\n", name, value);
#endif
}

static void
divx_scroll_scb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  int move = (int) call_data;
  Dimension length;
  float shown, top1, top2;
  char *name;
  float step;
  float range;
  name = XtName (XtParent (widget));

  XtVaGetValues (widget,
                 XtNlength, &length,
                 XtNshown, &shown, XtNtopOfThumb, &top1, NULL);

  if (strcmp(name, "divxwidth") == 0)
    {
      range = DIVX_WIDTH_RANGE;
      step = w_modul;
    }
  else if (strcmp(name, "divxheight") == 0)
    {
      range = DIVX_HEIGHT_RANGE;
      step = h_modul;
    }
  else if (strcmp(name, "divxbitrate") == 0)
    {
      range = DIVX_BITRATE_RANGE;
      step = 1.0;
    }
  /*
  else if (strcmp(name, "divxquality") == 0)
    {
      range = DIVX_QUALITY_RANGE;
      step = 1.0;
    }
  */
  else if (
           (strcmp(name, "divxminquantizer") == 0)
           ||(strcmp(name, "divxmaxquantizer") == 0)
           )
    {
      range = DIVX_MINQUANTIZER_RANGE;
      step = 1.0;
    }
  else if (strcmp(name, "divxaudiobufsize") == 0)
    {
      range = AUDIO_BUFFERSIZE_RANGE;
      step = 441.0;
    }
#ifdef HAVE_LAME
  else if (strcmp(name, "divxmp3bitrate") == 0)
    {
      range = MP3_BITRATE_RANGE;
      step = 16.0;
    }
  else if (strcmp(name, "divxmp3quality") == 0)
    {
      range = MP3_QUALITY_RANGE;
      step = 1.0;
    }
#endif // HAVE_LAME
  else
    {
      range = 65535.0;
      step = 5.0;
    }

  //top2 = top1 + (float) move / length / 5;
  if (move > 0)
    top2 = top1 + step/range;
  else
    top2 = top1 - step/range;
  if (top2 < 0.0)
    top2 = 0.0;
  if (top2 > 1.0)
    top2 = 1.0;

#if 0
  fprintf (stderr, "name: %s, range = %f, step = %f\n", name, range, step);
  fprintf (stderr, "top1 = %f, move = %d, step/range = %f, top2 = %f\n",top1,move,step/range,top2);
  fprintf (stderr, "scroll by %d\tlength %d\tshown %f\ttop %f => %f\n",
           move, length, shown, top1, top2);
#endif

  divx_jump_scb (widget, clientdata, &top2);
}

void
create_divxwin(void)
{
  Widget c,p;
  char label[64];
  XrmDatabase rb;
  char * divx_TranslationTable = "<Key>Escape:    DivX()";
  char * dir;

  rb = XtDatabase(XtDisplay(app_shell));

  XrmPutStringResource(&rb, "DivX.paned.internalBorderWidth",    "0");
  XrmPutStringResource(&rb, "DivX.paned*Scrollbar.orientation",  "horizontal");
  XrmPutStringResource(&rb, "DivX.paned*Scrollbar.length",       "150");
  XrmPutStringResource(&rb, "DivX.paned*Scrollbar.width",        "150");
  XrmPutStringResource(&rb, "DivX.paned*Scrollbar.minimumThumb", "3");
  XrmPutStringResource(&rb, "DivX*ldivxcodec.justify", "center");
  XrmPutStringResource(&rb, "DivX*ldivxsize.justify", "center");
  XrmPutStringResource(&rb, "DivX*ldivxbitrate.justify", "center");
  XrmPutStringResource(&rb, "DivX*ldivxquality.justify", "center");
  XrmPutStringResource(&rb, "DivX*ldivxaudio.justify", "center");
  XrmPutStringResource(&rb, "DivX*ldivxdisplay.justify", "center");
  XrmPutStringResource(&rb, "DivX*ldivxfilepath.justify", "center");

  XrmPutStringResource(&rb, "DivX*Label.shadowWidth",    "0");
  XrmPutStringResource(&rb, "DivX*Command.shadowWidth",  "0");
  XrmPutStringResource(&rb, "DivX*Command.translations", "#override\n<EnterWindow>: SetShadow(1)\n<LeaveWindow>: SetShadow(0)");
  XrmPutStringResource(&rb, "DivX*ldivxcodec.shadowWidth", "3");
  XrmPutStringResource(&rb, "DivX*ldivxsize.shadowWidth", "3");
  XrmPutStringResource(&rb, "DivX*ldivxbitrate.shadowWidth", "3");
  XrmPutStringResource(&rb, "DivX*ldivxquality.shadowWidth", "3");
  XrmPutStringResource(&rb, "DivX*ldivxaudio.shadowWidth", "3");
  XrmPutStringResource(&rb, "DivX*ldivxdisplay.shadowWidth", "3");
  XrmPutStringResource(&rb, "DivX*ldivxfilepath.shadowWidth", "3");
  //XrmPutStringResource(&rb, "DivX*Text*width", "120");
  XrmPutStringResource(&rb, "DivX*Text*autoFill", "false");
  XrmPutStringResource(&rb, "DivX*Text*editType", "edit");
  XrmPutStringResource(&rb, "DivX*Text*type", "string");
  XrmPutStringResource(&rb, "DivX*Text*useStringInPlace", "false");
  XrmPutStringResource(&rb, "DivX*Text*background", "white");
  XrmPutStringResource(&rb, "DivX*paned.resize", "false");
  XrmPutStringResource(&rb, "DivX*divxfilepath.translations", "#override\n<EnterWindow>: SetBg(lightyellow)\n<LeaveWindow>: SetBg(white)");

  divx_shell = XtVaAppCreateShell ("DivX", "Xawdecode",
                                   topLevelShellWidgetClass,
                                   dpy, XtNclientLeader, app_shell, NULL);

  XtOverrideTranslations (divx_shell, XtParseTranslationTable
                          ("<Message>WM_PROTOCOLS: DivX()"));

  divx_paned =
    XtVaCreateManagedWidget ("paned", panedWidgetClass, divx_shell, NULL);

  XtOverrideTranslations (divx_paned, XtParseTranslationTable
                          (divx_TranslationTable));

  /* DivX Codec Label */  
  c = XtVaCreateManagedWidget ("ldivxcodec", labelWidgetClass, divx_paned,
                           PANED_FIX, NULL);
  XtVaSetValues(c, XtNlabel, "Codec", NULL);

  /* Divx Codec Menu */
  c_divxcodec = c = XtVaCreateManagedWidget ("divxcodec", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_CODEC);
  sprintf (label, "%-" LABEL_WIDTH2 "s:       %s", "Codec",
           int_to_str (divx.codec, codec_list));
  quality_list = set_quality_list(divx.codec);
  XtVaSetValues (c, XtNlabel, label, NULL);
  
  /* DivX Size Label */  
  c = XtVaCreateManagedWidget ("ldivxsize", labelWidgetClass, divx_paned,
                           PANED_FIX, NULL);
  XtVaSetValues(c, XtNlabel, "Video Size", NULL);

  /* DivX W-Modul Slider */  
  c_wmodul = c = XtVaCreateManagedWidget ("wmodul", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_WMODUL);
  sprintf (label, "%-" LABEL_WIDTH2 "s: %5d", "W-Modul", w_modul);
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* DivX Width Slider */  
  p =
    XtVaCreateManagedWidget ("divxwidth", panedWidgetClass, divx_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxwidth = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxwidth = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxwidth, XtNtopOfThumb, (float)divx.width/DIVX_WIDTH_RANGE);
  sprintf(label, "%-" LABEL_WIDTH2 "s: %5d", "Width", divx.width);
  XtVaSetValues(l_divxwidth, XtNlabel, label, NULL);
  
  XtAddCallback (s_divxwidth, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxwidth, XtNscrollProc, divx_scroll_scb, NULL);


  /* DivX H-Modul Slider */  
  c_hmodul = c = XtVaCreateManagedWidget ("hmodul", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_HMODUL);
  sprintf (label, "%-" LABEL_WIDTH2 "s: %5d", "H-Modul", h_modul);
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* DivX Height Slider */  
  p =
    XtVaCreateManagedWidget ("divxheight", panedWidgetClass, divx_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxheight = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxheight = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxheight, XtNtopOfThumb, (float)divx.height/DIVX_HEIGHT_RANGE);
  sprintf(label, "%-" LABEL_WIDTH2 "s: %5d", "Height", divx.height);
  XtVaSetValues(l_divxheight, XtNlabel, label, NULL);

  XtAddCallback (s_divxheight, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxheight, XtNscrollProc, divx_scroll_scb, NULL);



  /* DivX Bitrate Label */  
  c = XtVaCreateManagedWidget ("ldivxbitrate", labelWidgetClass, divx_paned,
                           PANED_FIX, NULL);
  XtVaSetValues(c, XtNlabel, "Bitrate", NULL);

  /* DivX Bitrate Slider */  
  p =
    XtVaCreateManagedWidget ("divxbitrate", panedWidgetClass, divx_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxbitrate = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxbitrate = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxbitrate, XtNtopOfThumb, (float)divx.bitrate/1000/DIVX_BITRATE_RANGE);
  sprintf(label, "%-" LABEL_WIDTH2 "s: %5ld", "Bitrate (kbit/s)", divx.bitrate/1000);
  XtVaSetValues(l_divxbitrate, XtNlabel, label, NULL);

  XtAddCallback (s_divxbitrate, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxbitrate, XtNscrollProc, divx_scroll_scb, NULL);



  /* DivX Quality & Quantizers Label */  
  c = XtVaCreateManagedWidget ("ldivxquality", labelWidgetClass, divx_paned,
                           PANED_FIX, NULL);
  XtVaSetValues(c, XtNlabel, "DivX4 Quality & Quantizers", NULL);


  /* DivX Quality Menu */  
  c_divxquality = c = XtVaCreateManagedWidget ("divxquality", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_QUALITY);
  sprintf (label, "%-" LABEL_WIDTH2 "s:       %s", "Quality",
           int_to_str (divx.quality, quality_list));
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* DivX min Quantizer Slider */  
  p =
    XtVaCreateManagedWidget ("divxminquantizer", panedWidgetClass, divx_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxminquantizer = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxminquantizer = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxminquantizer, XtNtopOfThumb, (float)divx.min_quantizer/DIVX_MINQUANTIZER_RANGE);
  sprintf(label, "%-" LABEL_WIDTH2 "s: %5d", "Min Quantizer", divx.min_quantizer);
  XtVaSetValues(l_divxminquantizer, XtNlabel, label, NULL);

  XtAddCallback (s_divxminquantizer, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxminquantizer, XtNscrollProc, divx_scroll_scb, NULL);


  /* DivX max Quantizer Slider */  
  p =
    XtVaCreateManagedWidget ("divxmaxquantizer", panedWidgetClass, divx_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxmaxquantizer = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxmaxquantizer = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxmaxquantizer, XtNtopOfThumb, (float)divx.max_quantizer/DIVX_MAXQUANTIZER_RANGE);
  sprintf(label, "%-" LABEL_WIDTH2 "s: %5d", "Max Quantizer", divx.max_quantizer);
  XtVaSetValues(l_divxmaxquantizer, XtNlabel, label, NULL);

  XtAddCallback (s_divxmaxquantizer, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxmaxquantizer, XtNscrollProc, divx_scroll_scb, NULL);


  /* AVI Audio Label */  
  c = XtVaCreateManagedWidget ("ldivxaudio", labelWidgetClass, divx_paned,
                           PANED_FIX, NULL);
  XtVaSetValues(c, XtNlabel, "AVI Audio", NULL);

  /* Audio Buffer Size Slider */  
  p =
    XtVaCreateManagedWidget ("divxaudiobufsize", panedWidgetClass, divx_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxaudiobufsize = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxaudiobufsize = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxaudiobufsize, XtNtopOfThumb, (float)divx.audio_buffer_size/AUDIO_BUFFERSIZE_RANGE);
  sprintf(label, "%-" LABEL_WIDTH2 "s: %5d", "Audio Buffer Size", divx.audio_buffer_size);
  XtVaSetValues(l_divxaudiobufsize, XtNlabel, label, NULL);

  XtAddCallback (s_divxaudiobufsize, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxaudiobufsize, XtNscrollProc, divx_scroll_scb, NULL);

#ifdef HAVE_LAME
  /* Select compressed audio */
  c_compressaudio = c =
    XtVaCreateManagedWidget ("divxcompressedaudio", commandWidgetClass,
                             divx_paned, PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_COMPRESS_AUDIO);
  sprintf (label, "%-" LABEL_WIDTH2 "s:       %s", "Compress Audio",
           int_to_str (divx.compress_audio, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* MP3 Constant Bitrate Slider */  
  p_divxmp3bitrate = p =
    XtVaCreateManagedWidget ("divxmp3bitrate", panedWidgetClass, divx_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxmp3bitrate = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxmp3bitrate = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxmp3bitrate, XtNtopOfThumb, (float)divx.mp3_bitrate/MP3_BITRATE_RANGE);
  sprintf(label, "%-" LABEL_WIDTH2 "s: %5d", "MP3 Bitrate", divx.mp3_bitrate);
  XtVaSetValues(l_divxmp3bitrate, XtNlabel, label, NULL);

  XtAddCallback (s_divxmp3bitrate, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxmp3bitrate, XtNscrollProc, divx_scroll_scb, NULL);

  /* MP3 Quality Slider */  
  p_divxmp3quality = p =
    XtVaCreateManagedWidget ("divxmp3quality", panedWidgetClass, divx_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_divxmp3quality = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_divxmp3quality = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  set_float(s_divxmp3quality, XtNtopOfThumb, (float)divx.mp3_quality/MP3_QUALITY_RANGE);
  sprintf(label, "%-" LABEL_WIDTH2 "s: %5d", "MP3 Quality", divx.mp3_quality);
  XtVaSetValues(l_divxmp3quality, XtNlabel, label, NULL);

  XtAddCallback (s_divxmp3quality, XtNjumpProc, divx_jump_scb, NULL);

  XtAddCallback (s_divxmp3quality, XtNscrollProc, divx_scroll_scb, NULL);
#endif // HAVE_LAME

  /* Display Frames Label */  
  c = XtVaCreateManagedWidget ("ldivxdisplay", labelWidgetClass, divx_paned,
                           PANED_FIX, NULL);
  XtVaSetValues(c, XtNlabel, "Display Frames during Capture", NULL);

  /* DivX Display Frames Menu */  
  c_displayframes = c = XtVaCreateManagedWidget ("divxdisplay", commandWidgetClass, divx_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, divx_menu_cb, (XtPointer) DIVX_DISPLAY_FRAMES);
  sprintf (label, "%-" LABEL_WIDTH2 "s:       %s", "Display Frames",
           int_to_str (divx.display_frames, on_off_list));
  XtVaSetValues (c, XtNlabel, label, NULL);

  /* DivX file path */
  c = XtVaCreateManagedWidget ("ldivxfilepath", labelWidgetClass, divx_paned,
                           PANED_FIX, NULL);
  XtVaSetValues(c, XtNlabel, "DivX file directory", NULL);
  t_divxfilepath = c = XtVaCreateManagedWidget ("divxfilepath", asciiTextWidgetClass, divx_paned,
                                                PANED_FIX, NULL);
  //dir = getenv("HOME");
  dir = malloc(256);
  getcwd(dir, 255);
  XtVaSetValues (t_divxfilepath, XtNstring, dir, NULL);
  free(dir);
}

void
get_divxfilepath(char **name)
{
  XtVaGetValues (t_divxfilepath, XtNstring, name, NULL);
}
#endif // defined(HAVE_DIVX4LINUX) || defined(HAVE_FFMPEG) || defined (HAVE_XVID)
